# Core lyric_naver
# This file fetches html document from naver and parse it.
#
# ChangeLog:
# 07 Feb 2006: Fixed bug when fetched html does not have 
#     canonical title and artist. (by Yeom Jaehyun)
#
require 'net/http'
require 'uri'
require 'cgi'
require 'iconv'

module MyWeb
  # Fetch html document from url
  def MyWeb.fetchFromHttp(url)
    protocol, host, path = url.strip.scan(/([^\:]*:\/\/)([^\/]*)(.*)/)[0]
    h = Net::HTTP.new( host.untaint, 80, nil, nil)
    response = h.untaint.get(path)
    response.code == "200" ? response.body : nil
  end
end

module Naver
  # Convert query string from utf-8 to euc-kr which is naver's encoding
  def Naver.getQueryString(query)
    CGI.escape(Iconv.new('euc-kr', 'utf-8').iconv(CGI.unescape(query).delete('.')))
  end

  # Parse song id from fetched html
  def Naver.getSongId(query)
    queryStr = getQueryString(query)
    url = "http://music.naver.com/search.nhn?where=lyric&query=#{queryStr}"
    body = MyWeb.fetchFromHttp(url)
    scanned = body.scan(/javascript\:lyric\(\'([0-9]*)\'\)/)[0]
    scanned == nil ? nil : scanned[0]
  end

  # parse lyric from fetched html
  def Naver.parseLyric(body)
    scanned = body.scan(/>([^-]*) - <b>([^<]*)<\/b>/)
	title, artist = ["", ""]
	if scanned != nil && scanned[0] != nil then
      title = Iconv.new('utf-8', 'euc-kr').iconv(scanned[0][0])
      artist = Iconv.new('utf-8', 'euc-kr').iconv(scanned[0][1])
	end
    lyric = Iconv.new('utf-8', 'euc-kr').iconv(body.scan(/<textarea[^>]*>([^<]*)<\/textarea>/)[0][0])
    return title, artist, lyric
  end

  # Get lyric from specified url
  def Naver.getLyricFromUrl(url)
    body = MyWeb.fetchFromHttp(url)
	body != nil ? parseLyric(body) : [nil, nil, nil]
  end

  # Get lyric from query and songId is gotten if it is nil
  def Naver.getLyric(query, songId=nil)
    queryStr = getQueryString(query)
    songId = getSongId(query) if songId == nil
    return "" if songId == nil
    url = "http://music.naver.com/search.nhn?where=lyric&popup=lyric&songid=#{songId}&query=#{queryStr}"
    getLyricFromUrl(url)
  end

  # Get lyric and convert it to html format
  def Naver.getLyricHtml(query, songId=nil)
    title, artist, lyric = self.getLyric(query, songId)
    return "가사를 찾지 못했습니다." if title == nil or artist == nil or lyric == nil
    html = "<b>#{title}</b><br>\n<u>#{artist}</u><br><br>\n\n" + lyric.gsub(/[\r]*\n/, "<br>\n")
  end
end
